# Erecall CI

# Requires Version 0.5.0 of the recci package:
#
#   http://codalism.com/~wew/w12recci/
library("recci"); 

# Scenario:
#   - 1 million documents
#   - 5%  richness
#   - 75% recall
#   - 50% precision
#   - 1,500 control set sample size
#   - 1,500 null set sample size
# Therefore
#   - Collection yield: 50,000
#   - Production yield: 37,500
#   - Production size:  75,000
#   - Null set size:   925,000
#   - Null set yield:   12,500
scen.1 <- list(coll.sz=1000000, rich=0.05, rec=0.75, prec=0.5,
    coll.smpl.sz=1500, null.smpl.sz=1500);

# Fill out a scenario from the base information to derive statistics.
fill.scen <- function(scen) {
    scen$coll.yld <- scen$coll.sz * scen$rich;
    scen$prod.yld <- scen$coll.yld * scen$rec;
    scen$prod.sz <- scen$prod.yld / scen$prec;
    scen$null.sz <- scen$coll.sz - scen$prod.sz;
    scen$null.yld <- scen$coll.yld - scen$prod.yld;
    scen;
}

# Repeatedly sample from a scenario and calculate confidence intervals
# on recall using the eRecall method.
erec.smpl.ci <- function(scen, rep=1000) {
    scen <- fill.scen(scen);
    coll.pos.v <- rhyper(rep, scen$coll.yld, 
        scen$coll.sz - scen$coll.yld, scen$coll.smpl.sz);
    null.pos.v <- rhyper(rep, scen$null.yld,
        scen$null.sz - scen$null.yld, scen$null.smpl.sz);
    sapply(1:rep, function(r) {
        coll.pos <- coll.pos.v[r];
        null.pos <- null.pos.v[r];
        coll <- c(pop=scen$coll.sz, smpl=scen$coll.smpl.sz, pos=coll.pos);
        unretr <- c(pop=scen$null.sz, smpl=scen$null.smpl.sz, pos=null.pos);
        # erecci is provided by the recci package
        erecci(coll, unretr)
    });
}

# Repeatedly sample from a scenario and calculate confidence intervals
# on recall using the (by default koopman variant of the) segmented method,
# as implemented by the recci package and described by Webber, "Approximate
# Recall Confidence Intervals", ACMTOIS (2013).
rec.smpl.ci <- function(scen, rep=1000, meth="koopman") {
    scen <- fill.scen(scen);
    retr.smpl.sz <- ceiling(scen$null.smpl.sz * scen$prod.sz / scen$coll.sz);
    unretr.smpl.sz <- scen$null.smpl.sz - retr.smpl.sz;
    retr.pos.v <- rhyper(rep, scen$prod.yld, 
        scen$prod.sz - scen$prod.yld, retr.smpl.sz);
    unretr.pos.v <- rhyper(rep, scen$null.yld,
        scen$null.sz - scen$null.yld, unretr.smpl.sz);
    sapply(1:rep, function(r) {
        retr.pos <- retr.pos.v[r];
        unretr.pos <- unretr.pos.v[r];
        retr <- c(pop=scen$prod.sz, smpl=retr.smpl.sz, pos=retr.pos);
        unretr <- c(pop=scen$null.sz, smpl=unretr.smpl.sz, pos=unretr.pos);
        recci(retr, unretr, method=meth)
    });
}

# Repeatedly sample from a scenario and calculate confidence intervals
# using the direct method (i.e. an exact binomial CI on the proportion
# of relevant documents in the sample that have been retrieved).
dir.smpl.ci <- function(scen, rep=1000) {
    scen <- fill.scen(scen);
    retr.smpl.sz <- rhyper(rep, scen$prod.sz, scen$null.sz, scen$null.smpl.sz);
    unretr.smpl.sz <- scen$null.smpl.sz - retr.smpl.sz;
    retr.pos.v <- rhyper(rep, scen$prod.yld, scen$prod.sz - scen$prod.yld,
        retr.smpl.sz);
    unretr.pos.v <- rhyper(rep, scen$null.yld, scen$null.sz - scen$null.yld,
        unretr.smpl.sz);
    sapply(1:rep, function(r) {
        binom.test(retr.pos.v[r], retr.pos.v[r] + unretr.pos.v[r])$conf.int;
    });
}

# Do the sampling experiments for the three methods.
do.smpl <- function(rep=10000) {
    erec.ci <- erec.smpl.ci(scen.1, rep);
    dir.ci <- dir.smpl.ci(scen.1, rep);
    rec.ci <- rec.smpl.ci(scen.1, rep);
    # Betabin is not significantly narrower than Koopman on this scenario
    #bbin.ci <- rec.smpl.ci(scen.1, rep, "betabin.half");
    #list(erec=erec.ci, rec=rec.ci, dir=dir.ci, bbin=bbin.ci);
    list(erec=erec.ci, rec=rec.ci, dir=dir.ci);
}

# Calculate mean width across a data frame of CIs for a given method.
mn.wd <- function(ci, meth) {
    mean(ci[[meth]][2,] - ci[[meth]][1,]);
}

# Calculate width across a data frame of CIs for a given method.
ci.wd <- function(ci, meth) {
    ci[[meth]][2,] - ci[[meth]][1,];
}

# Calculate mean lower bound across a data frame of CIs for a given method.
mn.lo <- function(ci, meth) {
    mean(ci[[meth]][1,]);
}

# Calculate mean upper bound across a data frame of CIs for a given method.
#
# |ci| is an object of the form returned by the function |do.smpl()|
mn.hi <- function(ci, meth) {
    mean(ci[[meth]][2,]);
}

# Calculate summary statistics for all CI methods.
smy.smpl <- function(ci, meth.v=names(ci)) {
    wd <- sapply(meth.v, function(meth) mn.wd(ci, meth));
    lo <- sapply(meth.v, function(meth) mn.lo(ci, meth));
    hi <- sapply(meth.v, function(meth) mn.hi(ci, meth));
    df <- data.frame(wd=wd, lo=lo, hi=hi, row.names=meth.v);
    df;
}

# #################################################################
# Point estimates.
# 
# All methods are approximately unbiased.  Average estimate across
# 100,000 simulations are:
#
#                Mean   Std. Dev
#    eRec:      0.747    0.0635
#    direct:    0.751    0.0466
#    segmented: 0.751    0.0466
# #################################################################

erec.pt.est <- function(scen, rep=1000) {
    scen <- fill.scen(scen);
    coll.pos.v <- rhyper(rep, scen$coll.yld, 
        scen$coll.sz - scen$coll.yld, scen$coll.smpl.sz);
    null.pos.v <- rhyper(rep, scen$null.yld,
        scen$null.sz - scen$null.yld, scen$null.smpl.sz);
    coll.yld.est <- scen$coll.sz * coll.pos.v / scen$coll.smpl.sz;
    null.yld.est <- scen$null.sz * null.pos.v / scen$null.smpl.sz;
    prod.yld.est <- coll.yld.est - null.yld.est;
    prod.yld.est / coll.yld.est;
}

rec.pt.est <- function(scen, rep=1000) {
    scen <- fill.scen(scen);
    retr.smpl.sz <- ceiling(scen$null.smpl.sz * scen$prod.sz / scen$coll.sz);
    unretr.smpl.sz <- scen$null.smpl.sz - retr.smpl.sz;
    retr.pos.v <- rhyper(rep, scen$prod.yld, 
        scen$prod.sz - scen$prod.yld, retr.smpl.sz);
    unretr.pos.v <- rhyper(rep, scen$null.yld,
        scen$null.sz - scen$null.yld, unretr.smpl.sz);
    retr.yld.est <- scen$prod.sz * retr.pos.v / retr.smpl.sz;
    unretr.yld.est <- scen$null.sz * unretr.pos.v / unretr.smpl.sz;
    retr.yld.est / (retr.yld.est + unretr.yld.est);
}

dir.pt.est <- function(scen, rep=1000) {
    scen <- fill.scen(scen);
    retr.smpl.sz <- rhyper(rep, scen$prod.sz, scen$null.sz, scen$null.smpl.sz);
    unretr.smpl.sz <- scen$null.smpl.sz - retr.smpl.sz;
    retr.pos.v <- rhyper(rep, scen$prod.yld, scen$prod.sz - scen$prod.yld,
        retr.smpl.sz);
    unretr.pos.v <- rhyper(rep, scen$null.yld, scen$null.sz - scen$null.yld,
        unretr.smpl.sz);
    retr.yld.est <- scen$prod.sz * retr.pos.v / retr.smpl.sz;
    unretr.yld.est <- scen$null.sz * unretr.pos.v / unretr.smpl.sz;
    retr.yld.est / (retr.yld.est + unretr.yld.est);
}
